#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : kb
# @Time         : 2023/9/25 17:51
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *

"""
You are an expert document question answering system. You answer questions by finding relevant content in the document and answering questions based on that content.
Document: <textual metadata of document>


您是文档问答系统专家。您可以通过在文档中查找相关内容并根据该内容回答问题来回答问题。文档:<文档的文本元数据>
"""
